/**
 * Copyright (C) 2012 Ness Computing, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.likeness.lifecycle.guice;


import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.google.inject.Singleton;
import com.likeness.lifecycle.Lifecycle;

/**
 * Annotate a method to be hooked up to a certain lifecycle stage.
 * The annotated method will be invoked as if it were added as a listener via the normal {@link Lifecycle#addListener}.
 * The method must not take arguments, and should not return a value.
 * All instances of your enclosing class must be created before the lifecycle actually starts.
 * In practice this usually means that this should only be used on {@link Singleton} classes.
 */
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
public @interface OnStage {
    String value();
}
